<!doctype html>
<HTML>
<HEAD>
  <meta charset="utf-8" />
  <TITLE>Jackdaw</TITLE>

  <link rel="stylesheet" type="text/css" href="/nest/static/css/bootstrap.min.css">
  <link rel="stylesheet" type="text/css" href="/nest/static/css/vis.min.css">

  <script type="text/javascript" src="/nest/static/js/vis.min.js"></script>
  <script type="text/javascript" src="/nest/static/js/jquery-3.4.1.min.js"></script>
  <script type="text/javascript" src="/nest/static/js/graph.js"></script>
  <script type="text/javascript" src="/nest/static/js/anomalies.js"></script>

  <script src="/nest/static/js/popper.min.js"></script>
  <script src="/nest/static/js/bootstrap.min.js"></script>



  <style type="text/css">
    #mynetwork {
        /* width: 600px; */
        width: 100%;
        height: 800px;
        border: 2px solid lightgray;
    }
    </style>

</HEAD>

<BODY>

    <div class="container-fluid">
        <div class="row">
          <div class="col-md-12">
            <nav class="navbar navbar-expand-lg navbar-light bg-light">
               
              <button class="navbar-toggler" type="button" data-toggle="collapse" data-target="#bs-example-navbar-collapse-1">
                <span class="navbar-toggler-icon"></span>
              </button><a href="#" class="navbar-left"><img style="max-width:80px; margin-top: -7px;" src="nest/static/img/jackdaw_icon_big.png"></a>
              <div class="collapse navbar-collapse" id="bs-example-navbar-collapse-1">
                <ul class="navbar-nav">
                  <li class="nav-item dropdown">
                      <a class="nav-link dropdown-toggle" id="navbarDropdownMenuLink" data-toggle="dropdown">Domain</a>
                     <div class="dropdown-menu" aria-labelledby="navbarDropdownMenuLink">
                       <a class="dropdown-item" href="#">List</a>
                       <div class="dropdown-divider"> </div>
                       <a class="dropdown-item" href="#">User</a> 
                       <a class="dropdown-item" href="#">Machine</a>
                    </div>
                  </li>
                  <li class="nav-item dropdown">
                      <a class="nav-link dropdown-toggle" id="navbarDropdownMenuLink" data-toggle="dropdown">Anomalies</a>
                     <div class="dropdown-menu" aria-labelledby="navbarDropdownMenuLink">
                       <a class="dropdown-item" onclick="get_user_anomalies();">User</a> 
                       <a class="dropdown-item" href="#">Machine</a>
                     </div>
                  </li>
                  <li class="nav-item dropdown">
                     <a class="nav-link dropdown-toggle" id="navbarDropdownMenuLink" data-toggle="dropdown">Graph menu</a>
                    <div class="dropdown-menu" aria-labelledby="navbarDropdownMenuLink">
                      <a class="dropdown-item" href="#">Calculate</a>
                      <div class="dropdown-divider"> </div> 
                      <a class="dropdown-item" href="#">List</a> 
                      <a class="dropdown-item" href="#">Plot</a>
                    </div>
                  </li>
                </ul>
                <ul class="navbar-nav ml-md-auto">
                  <li class="nav-item active">
                     <a class="nav-link" href="#">Link <span class="sr-only">(current)</span></a>
                  </li>
                  <li class="nav-item dropdown">
                     <a class="nav-link dropdown-toggle" href="http://example.com" id="navbarDropdownMenuLink" data-toggle="dropdown">Dropdown link</a>
                    <div class="dropdown-menu dropdown-menu-right" aria-labelledby="navbarDropdownMenuLink">
                       <a class="dropdown-item" href="#">Action</a> <a class="dropdown-item" href="#">Another action</a> <a class="dropdown-item" href="#">Something else here</a>
                      <div class="dropdown-divider">
                      </div> <a class="dropdown-item" href="#">Separated link</a>
                    </div>
                  </li>
                </ul>
              </div>
            </nav>
          </div>
        </div>
        <div class="row" id="graphrow">
          <div class="col-md-2">
            <form role="form">
              <div class="form-group"> 
                <label for="graphid">
                  GRAPH ID
                </label>
                <input type="text" class="form-control"  required="" placeholder="Enter Graph ID. via Swagger only :(" id="graphid" />
              </div>
            </form>
            <form role="form">
              <div class="form-group"> 
                <label for="srcsid">
                  SRC SID
                </label>
                <input type="text" class="form-control" id="srcsid" />
              </div>
              <div class="form-group">
                <label for="dstsid">
                  DST SID
                </label>
                <input type="text" class="form-control" id="dstsid" />
              </div>
              <button type="button" class="btn btn-primary" onclick="do_graph('path');">
                DRAW PATH
              </button>
              <button type="button" class="btn btn-primary" onclick="do_graph('domainadmins');">
                DRAW DOMAIN ADMINS
              </button>
            </form>
          </div>
          <div class="col-md-10" id="grapharea">
          </div>
        </div>

        <div class="row" id="anomalies_user_row" style="visibility: hidden">
          <div class="col-md-12">
              <table class="table">
                <thead>
                  <th scope="col">username</th>
                  <th scope="col">sid</th>
                </thead>
                <tbody id="anomalies_user_pwnotexp_table">
                  
                </tbody>
              </table>
          </div>
        
        </div>
      </div>


</BODY>
</HTML>